package ovh.herisson.Clyde.Services.ScientificPublications;

import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.ScientificPublications.ResearchCoAuthorsRepository;
import ovh.herisson.Clyde.Repositories.ScientificPublications.ResearchRepository;
import ovh.herisson.Clyde.Repositories.ScientificPublications.ResearcherRepository;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.ScientificPublications.Access;
import ovh.herisson.Clyde.Tables.ScientificPublications.PaperType;
import ovh.herisson.Clyde.Tables.ScientificPublications.Research;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;
import ovh.herisson.Clyde.Tables.User;

import java.util.Date;
import java.util.Map;

@Service
@AllArgsConstructor
public class ResearchesService {

    private final ResearcherRepository researcherRepo;
    private final ResearchCoAuthorsRepository researchCoAuthorsRepo;
    private final ResearchRepository articleRepo;

    // Researchers Part
    public Researcher getResearcherByUser(User user){
        return researcherRepo.findByUser(user);
    }

    public boolean isCoAuthor(Research research, Researcher researcher){
        Iterable<Researcher> coAuthors = researchCoAuthorsRepo.findResearchCoAuthors(research);
        for (Researcher coAuthor : coAuthors){
            if (researcher == coAuthor){
                return true;
            }
        }
        return false;
    }


    // researches Part
    public Research getResearchById(long id) {
        return articleRepo.findById(id);
    }

    public boolean hasNoAccessTo(Research research, User user){

        if (research.getAccess() == Access.OpenSource) return false; // if the access is open source even non-users can see it
        if (user == null) return true; // else you need at least to be a user

        if (research.getAccess() == Access.Restricted &&
                user.getRole() == Role.Admin || user.getRole() == Role.Secretary ||
                user.getRole() == Role.Teacher || user.getRole() == Role.InscriptionService)
            return false; // if the access is restricted only the staff member (above) can access the research

        Researcher researcher = getResearcherByUser(user);

        if (researcher==null)
            return true;

        return (research.getAccess() != Access.Private || research.getAuthor() != researcher) &&
                (research.getAccess() != Access.Private || !isCoAuthor(research, researcher));
        // if the researcher is the author or one of the co-authors of the research will return false
    }

    public Research saveResearch(Research research) {
        return articleRepo.save(research);
    }

    public void modifyResearchData(Research research, Map<String, Object> updates) {
        for (Map.Entry<String, Object> entry : updates.entrySet()){
            switch (entry.getKey()){
                case "title":
                    research.setTitle((String) entry.getValue());
                    break;
                case "date":
                    research.setReleaseDate((Date) entry.getValue());
                    break;
                case "paperType":
                    research.setPaperType((PaperType) entry.getValue());
                    break;
                case "PdfLocation":
                    research.setPdfLocation((String) entry.getValue());
                    break;
                case "bibtexLocation":
                    research.setBibTexLocation((String) entry.getValue());
                    break;
                case "language":
                    research.setLanguage((String) entry.getValue());
                    break;
                case "domain":
                    research.setDomain((String) entry.getValue());
                    break;
                case "summary":
                    research.setSummary((String) entry.getValue());
                    break;
            }
        }
        articleRepo.save(research);
    }

    public void deleteResearch(Research research) {
        articleRepo.delete(research);
    }

    public Iterable<Research> getAllResearches() {
        return articleRepo.findAll();
    }

    public Researcher saveResearcher(Researcher researcher) {
       return researcherRepo.save(researcher);
    }

    public Iterable<Researcher> getAllResearchers() {
        return researcherRepo.findAll();
    }

    public Researcher getResearcherById(long id) {
        return  researcherRepo.findById(id);
    }

    public void deleteResearcher(Researcher researcher) {
       researcherRepo.delete(researcher);
    }
}
